function [Intersections] = Trochoid_OuterLobe(theta_dot, R_cp, R_t, Phi_dot, nu, push_far_factor, push_close_factor, t_step_init_Factor)

%!!!!! - This code uses parallel processes - !!!!!

% Finds the outermost intersection points for the following parametric function:

% X(t) = R_cp * cos(Phi_dot * t) + R_t * cos(theta_dot * (-t))
% Y(t) =  R_cp * sin(Phi_dot * t) + R_t * sin(theta_dot * (-t)) + nu * t

% code runs efficienty by only checking points between an outer ring,
% defined by push_far_factor, and an inner ring, defined by
% push_close_factor. t_step_init_Factor adjust the spacing between points
% that are checked for intersections.

% push_far_factor should be >1
% push_close_factor should <1
% t_step_init_Factor is generally a value between 5 and 30, but should be
% set as low as possible for optimal efficiency

% push_far_factor, push_close_factor, and t_step_init_Factor can be defined
% iteratively. After observing the solution plot, there should be rings (red
% dashed line) located just outside of the outer lobe and just inside the
% outer lobe. If the plot looks choppy (i.e. low resolution) or points have been missed, the
% t_step_init_Factor should be increased.



warning('off','all');

%***if intersections are not identified, try reducing t_step_init**********

% Example parameters - extreme
% push_far_factor = 1.2;
% push_close_factor = 0.9865;
% theta_dot = 8000 * 2 * pi / 60;
% R_cp = 12;
% R_t = 5; 
% Phi_dot = 1;
% nu = 2;
% t_step_init_Factor = 26.401;

% Example parameters - normal
% push_far_factor = 1.2;
% push_close_factor = 0.95;
% theta_dot = 500 * 2 * pi / 60;
% R_cp = 12;
% R_t = 5; 
% Phi_dot = 1;
% nu = 2;
% t_step_init_Factor = 5;

% Code
plot_angle_degree = 45;
plot_angle = plot_angle_degree * pi / 180;
plot_angle_degree2 = 45;

plot_angle2 = plot_angle_degree2 * pi / 180;
clear_loops = 1;
check_loops = 1.25;
push_loops = 1;
plot_points = 20; %number of points to plot between arcs (display purpose only)

start = 0;
finish = pi;
t_step_init = t_step_init_Factor*theta_dot^(-1.984);
accuracy = 0.00001; %desired accuracy - points will be accurate
%to within + or - this value (XY, not time)

t_step_mult = 10*t_step_init;
push_close = push_close_factor*(R_t+R_cp);
push_far = push_far_factor*(R_t+R_cp);

%governing equations
X = @(t) R_cp * cos(Phi_dot * t) + R_t * cos(theta_dot * (-t));
Y = @(t) R_cp * sin(Phi_dot * t) + R_t * sin(theta_dot * (-t)) + nu * t ;
Y_stat = @(t) R_cp * sin(Phi_dot * t) + R_t * sin(theta_dot * (-t));
m_calc = @(t) (R_cp * Phi_dot * cos(Phi_dot * (t + t_step_init/2)) - R_t * theta_dot * cos(-theta_dot *(t + t_step_init/2)) + nu)...
    / (-R_cp * Phi_dot * sin(Phi_dot * (t + t_step_init/2)) + R_t * theta_dot * sin(-theta_dot *(t + t_step_init/2)));
m_calc2 = @(t) (R_cp * Phi_dot * cos(Phi_dot * (t)) - R_t * theta_dot * cos(-theta_dot *(t)) + nu)...
    / (-R_cp * Phi_dot * sin(Phi_dot * (t)) + R_t * theta_dot * sin(-theta_dot *(t)));
b_calc = @(t) Y(t) -  m_calc(t)*X(t);
X_push_far = @(t) (R_cp+R_t*push_far_factor)*cos(Phi_dot*t);
Y_push_far = @(t) (R_cp+R_t*push_far_factor)*sin(Phi_dot*t)+ nu*t;
X_push_close = @(t) (R_cp+R_t*push_close_factor)*cos(Phi_dot*t);
Y_push_close = @(t) (R_cp+R_t*push_close_factor)*sin(Phi_dot*t)+ nu*t;
Y_push_close_stat = @(t) (R_cp+R_t*push_close_factor)*sin(Phi_dot*t);
clear push_far_factor push_close_factor
push_begin = -4*pi/theta_dot;

%initial plot quickly shows up to check path
t_plot = start:t_step_init:finish;
t_plot_push = push_begin:t_step_mult:finish-push_begin;
clear push_begin t_step_mult
figure('Name','Trochoid Outer Lobe','Units','pixels','Position',[50 50 1250 1250])
axis equal
hold on
plot(X(t_plot),Y(t_plot),'-b');
plot(X_push_far(t_plot_push),Y_push_far(t_plot_push),'--r');
plot(X_push_close(t_plot_push),Y_push_close(t_plot_push),'--r');
clear Y_push_close Y_push_far X_push_far
drawnow

num_loops = floor(theta_dot/2);
j=1;
check_points = zeros(ceil((finish-start)/t_step_init),2);

parfor t_step=1:ceil((finish-start)/t_step_init)
    t=t_step*t_step_init;
     push_close = sqrt(X_push_close(t)^2+Y_push_close_stat(t)^2);
    dist_point = sqrt(X(t)^2+Y_stat(t)^2);
    if dist_point > push_close
        check_points(t_step,1) = t;
    end
end
clear Y_push_close_stat Y_stat

n=1;
p=1;
j=1;
while n<size(check_points,1)
    if check_points(n,1) == 0
        if j>1
            p=p+1;
            j=1;
        end
    else
        time_array{p,j} = check_points(n,1);
        j=j+1;
    end
    check_points(n,:) = [];
end

clear check_points n p j

intersect1 = zeros(size(time_array,1)-1,1);
intersect2 = zeros(size(time_array,1)-1,1);

for loop = 2:size(time_array,1)-1
    for t_step=1:size(time_array,2)
        t=cell2mat(time_array(loop,t_step));
        if isempty(t) || intersect1(loop) ~= 0
            break
        end
        for t_check_step = 1:size(time_array,2)
            t_check = cell2mat(time_array(loop+1,t_check_step));
            if isempty(t_check)
                break
            end
            X_i = (b_calc(t)-b_calc(t_check))/...
                (m_calc(t_check)-m_calc(t)); %calculates intersection point x
            Y_i = m_calc(t)* X_i + b_calc(t); %calculates intersection point y
            if (((X_i < X(t) && X_i > X(t+t_step_init)) | ...
                    (X_i > X(t) && X_i < X(t+t_step_init))) && ...
                    ((Y_i < Y(t) && Y_i > Y(t+t_step_init)) | ...
                    (Y_i > Y(t) && Y_i < Y(t+t_step_init)))) && ...
                    (((X_i < X(t_check) && X_i > X(t_check+t_step_init)) | ...
                    (X_i > X(t_check) && X_i < X(t_check+t_step_init))) && ...
                    ((Y_i < Y(t_check) && Y_i > Y(t_check+t_step_init)) | ...
                    (Y_i > Y(t_check) && Y_i < Y(t_check+t_step_init))))
                int_points1 = t;
                int_points2 = t_check;
                t_step_loop1 = t_step_init;
                t_step_loop2 = t_step_init;
                gap_curr = sqrt((X(int_points1)-X(int_points1+t_step_loop1))^2 ...
                    +(Y(int_points1)-Y(int_points1+t_step_loop1))^2);
                gap_pre = sqrt((X(int_points2)-X(int_points2+t_step_loop2))^2 ...
                    +(Y(int_points2)-Y(int_points2+t_step_loop2))^2);
                %loops until the gap between points is smaller than the
                %accuracy value
                while gap_curr > accuracy || gap_pre >accuracy
                    X_i = ((Y(int_points1)-m_calc2(int_points1+t_step_loop1/2)*X(int_points1))...
                        -(Y(int_points2)-m_calc2(int_points2+t_step_loop2/2)*X(int_points2)))/...
                        (m_calc2(int_points2+ t_step_loop2/2)-m_calc2(int_points1+ t_step_loop1/2));
                    Y_i = m_calc2(int_points1+ t_step_loop1/2)* X_i + ...
                        (Y(int_points1)-m_calc2(int_points1+t_step_loop1/2)*X(int_points1)); %calculates intersection point y
                    dist_curr_1 = sqrt((X(int_points1)-X_i)^2 ...
                        +(Y(int_points1)-Y_i)^2);
                    dist_curr_2 = sqrt((X(int_points1+t_step_loop1)-X_i)^2 ...
                        +(Y(int_points1+t_step_loop1)-Y_i)^2);
                    dist_pre_1 = sqrt((X(int_points2)-X_i)^2 ...
                        +(Y(int_points2)-Y_i)^2);
                    dist_pre_2 = sqrt((X(int_points2+t_step_loop2)-X_i)^2 ...
                        +(Y(int_points2+t_step_loop2)-Y_i)^2);
                    int_points1 = int_points1 + (t_step_loop1*dist_curr_1/gap_curr)/2;
                    t_step_loop1 = t_step_loop1-t_step_loop1*dist_curr_2/gap_curr/2;
                    int_points2 = int_points2 + (t_step_loop2*dist_pre_1/gap_pre)/2;
                    t_step_loop2 = t_step_loop2-t_step_loop2*dist_pre_2/gap_pre/2;
                    gap_curr = sqrt((X(int_points1)-X(int_points1+t_step_loop1))^2 ...
                        +(Y(int_points1)-Y(int_points1+t_step_loop1))^2);
                    gap_pre = sqrt((X(int_points2)-X(int_points2+t_step_loop2))^2 ...
                        +(Y(int_points2)-Y(int_points2+t_step_loop2))^2);
                end
                intersect1(loop-1) =   int_points1;
                intersect2(loop-1) = int_points2;
                break
            end
        end
    end
end
clear b_calc m_calc2 m_calc time_array

intersect1(size(intersect1,1),:) = [];
intersect2(size(intersect2,1),:) = [];

plot(X(intersect1(:)),Y(intersect1(:)),'+','linestyle','none','color','red');
for i=1:num_loops-2
    plot(X(linspace(intersect1(i+1),intersect2(i),plot_points)),Y(linspace(intersect1(i+1),intersect2(i),plot_points)),'LineWidth',2);
end

Intersections = [intersect1, intersect2];
